window.Gleam.initSearch([{"doc":"effect","title":"effect","content":"&lt;h1 align=&quot;center&quot;&gt;Effect&lt;/h1&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  &lt;strong&gt;effectual computations made easy&lt;/strong&gt;\n&lt;/div&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  &lt;a href=&quot;https://hex.pm/packages/effect&quot;&gt;\n    &lt;img src=&quot;https://img.shields.io/hexpm/v/effect&quot; alt=&quot;Package Version&quot;/&gt;\n  &lt;/a&gt;\n  &lt;a href=&quot;https://hexdocs.pm/effect/&quot;&gt;\n    &lt;img src=&quot;https://img.shields.io/badge/hex-docs-ffaff3&quot; alt=&quot;Docs&quot;/&gt;\n  &lt;/a&gt;\n&lt;/div&gt;\n\n&lt;br/&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n    &lt;img src=&quot;https://raw.githubusercontent.com/ethanthoma/effect/refs/heads/main/image.webp&quot; alt=&quot;meme&quot;&gt;\n&lt;/div&gt;\n\n## Table of Contents\n\n1. [Overview](#overview)  \n1. [Example](#example)  \n1. [Installation](#installation)  \n1. [Usage](#usage)\n    - [Basic Construction](#basic-construction)  \n    - [Chaining Effects](#chaining-effects)  \n    - [Handling Promises](#handling-promises)\n    - [Pure Effects](#pure-effects)\n\n## Overview\n\nA small library for handling side effects! Particularly promises in Gleam.\n\nInspired by [Lustre](https://github.com/lustre-labs/lustre)&#39;s Effects and [effect-ts](https://github.com/Effect-TS/core).\n\nThis library allows you to treat async operations and potential failures as \n**first-class effectful computations**, which can be composed before finally being executed.\n\nThe motivation was to create an API for dealing with promises from [gleam_promises](https://hexdocs.pm/gleam_javascript/gleam/javascript/promise.html).\n\n## Example\n\n```gleam\nimport gleam/fetch\nimport gleam/io\nimport gleam/javascript/promise\nimport gleam/result\nimport gleam/string\n\nimport effect.{type Effect}\nimport effect/promise as effect_promise\n\npub type Error {\n  UriParse\n  Fetch(fetch.FetchError)\n}\n\npub fn main() {\n  let google: Effect(String, Error) = {\n    use uri &lt;- effect.from_result_map_error(\n      uri.parse(&quot;https://www.google.com&quot;),\n      fn (_) { UriParse }, // replacing the error\n    )\n    use req &lt;- effect.from_result_map_error(\n      request.from_uri(uri),\n      fn (_) { UriParse }\n    )\n    use resp &lt;- effect_promise.from_promise_result(fetch.send(req), Fetch)\n    use text &lt;- effect_promise.from_promise_result(\n      fetch.read_text_body(resp),\n      Fetch,\n    )\n    text.body |&gt; effect.continue\n  }\n\n  use res: Result(String, Error) &lt;- effect.perform(google)\n  case res {\n    Ok(body) -&gt; body |&gt; io.println\n    Error(e) -&gt; e |&gt; string.inspect |&gt; io.println_error\n  }\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/effect&gt;.\n\n## Installation\n\n```sh\ngleam add effect@2\n```\n\n## Usage\n\nBelow are demonstrations of common usages demonstrating on how to create and compose effects, \nhandle failures, work with promises, and actually `perform` the effect.\n\n### Basic Construction\n\n```gleam\nimport gleam/int\nimport gleam/io\n\nimport effect.{type Effect}\n\npub fn main() {\n  // Succeed / Fail shortcuts\n  // Effect(Int, early)\n  let eff_ok = effect.continue(42)\n  // Effect(msg, String)\n  let eff_err = effect.throw(&quot;Oops!&quot;)\n\n  // Combine\n  let eff: Effect(Int, String) = {\n    case int.random(2) |&gt; int.is_even {\n      True -&gt; eff_ok\n      False -&gt; eff_err\n    }\n  }\n\n  // Perform\n  use res: Result(Int, String) &lt;- effect.perform(eff)\n\n  case res {\n    Ok(num) -&gt; io.println(&quot;Got: &quot; &lt;&gt; int.to_string(num))\n    Error(err) -&gt; io.println(&quot;Error: &quot; &lt;&gt; err)\n  }\n}\n```\n\n### Chaining Effects\n\nUse `then` and `map` to sequence operations, only continuing on success, or short-circuiting on error:\n\n```gleam\nimport gleam/float\nimport gleam/int\nimport gleam/io\n\nimport effect.{type Effect}\n\ntype TooSmallError {\n  TooSmallError\n}\n\npub fn main() {\n  let computation: Effect(Int, TooSmallError) = {\n    let effect: Effect(Int, TooSmallError) = effect.continue(10)\n    use num: Int &lt;- effect.then(effect)\n    case num &lt; 5 {\n      True -&gt; effect.throw(TooSmallError)\n      False -&gt; effect.continue(num * 2)\n    }\n  }\n\n  let computation: Effect(Float, TooSmallError) = {\n    use num: Int &lt;- effect.map(computation)\n    let f: Float = int.to_float(num)\n    // fun fact, I added this func to std\n    let log: Float = float.exponential(f)\n  }\n\n  use res: Result(Float, TooSmallError) &lt;- effect.perform(computation)\n  case res {\n    Ok(n) -&gt; io.println(&quot;Final result: &quot; &lt;&gt; float.to_string(n))\n    Error(TooSmallError) -&gt; io.println(&quot;Error: num is too small!&quot;)\n  }\n}\n```\n\n### Handling Promises\n\nYou can convert a `promise.Promise(Result(a, e))` into an `Effect(a, e)` using \n`from_promise_result`.\n\n```gleam\nimport gleam/fetch.{type FetchBody, type FetchError}\nimport gleam/javascript/promise.{type Promise}\nimport gleam/string\n\nimport effect.{type Effect}\nimport effect/promise as effect_promise\n\ntype Error {\n  Fetch(fetch.FetchError)\n}\n\nfn fetch_data() -&gt; Promise(Result(FetchBody, FetchError)) {\n  let assert Ok(uri) = uri.parse(&quot;https://www.google.com&quot;)\n  let assert Ok(req) = request.from_uri(uri)\n  fetch.send(req)\n}\n\nfn main() {\n  let eff: Effect(String, Error) = {\n    // fetch from google\n    let prom = fetch_data_readme()\n    // get an effect from a promise(result)\n    use fetch_body: Response(FetchBody) &lt;- effect_promise.from_promise_result(\n      prom,\n      Fetch, // map the error\n    )\n\n    // read the response from FetchBody to String\n    let prom: Promise(Result(Response(String), FetchError)) =\n      fetch.read_text_body(fetch_body)\n\n    use text: Response(String) &lt;- effect_promise.from_promise_result(\n      prom,\n      // map the error\n      Fetch,\n    )\n\n    // return just the body\n    text.body |&gt; effect.continue\n  }\n\n  use res: Result(String, Error) &lt;- effect.perform(eff)\n  case res {\n    // Server Data\n    Ok(body) -&gt; io.println(body)\n    // Network Error\n    Error(Fetch(msg)) -&gt; io.println(&quot;Failed: &quot; &lt;&gt; string.inspect(msg))\n    _ -&gt; io.println(&quot;Failed: other error&quot;)\n  }\n}\n```\n\n### Pure Effects\n\nSometimes you don&#39;t need the `early` return path and only need to operate on the \nhappy path. The `perform` function returns a `Result` type but there&#39;s a `pure` \nalternative:\n\n```gleam\nimport gleam/int\nimport gleam/io\n\nimport effect.{type Effect}\n\nfn main() {\n  let eff: Effect(Int, effect.Nothing) = {\n    use a &lt;- effect.from(5)\n    use b &lt;- effect.from(2)\n    a * b |&gt; effect.continue\n  }\n\n  use num: Int &lt;- effect.pure(eff)\n  num |&gt; int.to_string |&gt; io.println\n}\n```\n\nFor more, be sure to checkout the [test cases](./test/effect_test.gleam).\n","url":"index.html"},{"doc":"effect","title":"Effect","content":"pub opaque type Effect(msg, early)\n The `Effect` type represents computations that might return early or continue with a value.\n Each effect specifies:\n\n 1. The type of the continue value (`msg`)\n 2. The type of early return value (`early`)\n\n You can treat `msg` as the happy path.\n The `early` type is for long jumping.\n\n\nSynonyms:\neffect.Effect\neffect Effect","url":"effect.html#Effect"},{"doc":"effect","title":"Next","content":"pub type Next(msg) =\n  fn(msg) -&gt; Nil\n A function that handles the successful path of an effect.\n\n\nSynonyms:\neffect.Next\neffect Next","url":"effect.html#Next"},{"doc":"effect","title":"Not","content":"pub type Not(early) =\n  fn(early) -&gt; Nil\n A function that handles the early return path of an effect.\n\n\nSynonyms:\neffect.Not\neffect Not","url":"effect.html#Not"},{"doc":"effect","title":"Nothing","content":"pub type Nothing\n Type representing the absence of an early return value.\n\nSynonyms:\neffect.Nothing\neffect Nothing","url":"effect.html#Nothing"},{"doc":"effect","title":"continue","content":"pub fn continue(value: a) -&gt; Effect(a, b)\n Creates an effect that succeeds with the given value.\n\n \n let effect: Effect(Int, early) = continue(42)\n \nSynonyms:\neffect.continue\neffect continue","url":"effect.html#continue"},{"doc":"effect","title":"from","content":"pub fn from(\n  value: a,\n  handler: fn(a) -&gt; Effect(b, c),\n) -&gt; Effect(b, c)\n Creates an effect from a value with a handler to operate on it.\n\n \n use n: Int &lt;- from(5)\n \nSynonyms:\neffect.from\neffect from","url":"effect.html#from"},{"doc":"effect","title":"from_box","content":"pub fn from_box(\n  box: a,\n  unbox_fn: fn(a, fn(b) -&gt; Nil) -&gt; c,\n  handler: fn(b) -&gt; Effect(d, e),\n) -&gt; Effect(d, e)\n Creates an effect from a boxed value (like a Promise), using the provided unboxing \n function (like promise.map) and operates on the unboxed value via the handler.\n\n \n let promise: Promise(Result(ok, err))\n use result: Result(ok, err) &lt;- from_box(promise, promise.map)\n use ok: ok &lt;- from_result(result)\n \nSynonyms:\neffect.from_box\neffect from_box","url":"effect.html#from_box"},{"doc":"effect","title":"from_option","content":"pub fn from_option(\n  value: Option(a),\n  early: b,\n  handler: fn(a) -&gt; Effect(c, b),\n) -&gt; Effect(c, b)\n Creates an effect from an Option, where Some values are passed to the given function\n and None causes an early return.\nSynonyms:\neffect.from_option\neffect from_option","url":"effect.html#from_option"},{"doc":"effect","title":"from_result","content":"pub fn from_result(\n  value: Result(a, b),\n  handler: fn(a) -&gt; Effect(c, b),\n) -&gt; Effect(c, b)\n Creates an effect from a Result, where Ok values are passed to the given function\n and Error values cause an early return.\n\n \n let str: String = &quot;123&quot;\n let result: Result(Int, Nil) = parse_int(str)\n use num: Int &lt;- from_result(result)\n \nSynonyms:\neffect.from_result\neffect from_result","url":"effect.html#from_result"},{"doc":"effect","title":"from_result_map_error","content":"pub fn from_result_map_error(\n  value: Result(a, b),\n  map_error: fn(b) -&gt; c,\n  handler: fn(a) -&gt; Effect(d, c),\n) -&gt; Effect(d, c)\n Creates an effect from a Result, where Ok values are passed to the given function\n and Error values cause an early return.\n\n Maps the error using the given map_error for the final effect. \n Helper combines from_result and result.map_error\nSynonyms:\neffect.from_result_map_error\neffect from_result_map_error","url":"effect.html#from_result_map_error"},{"doc":"effect","title":"handle","content":"pub fn handle(\n  effect: Effect(a, b),\n  handler: fn(Result(a, b)) -&gt; Effect(c, d),\n) -&gt; Effect(c, d)\n Handles both paths of an effect, allowing transformation into a new effect\n with potentially different types.\n\n \n type ShowState {\n   ShowSuccess(valid)\n   ShowError(err)\n }\n \n use res: Result(valid, err) &lt;- handle(validate_input(data))\n case res {\n   Ok(valid) -&gt; continue(ShowSuccess(valid))\n   Error(err) -&gt; continue(ShowError(err))\n }\n \nSynonyms:\neffect.handle\neffect handle","url":"effect.html#handle"},{"doc":"effect","title":"map","content":"pub fn map(\n  effect: Effect(a, b),\n  handler: fn(a) -&gt; c,\n) -&gt; Effect(c, b)\n Transforms the successful values of an effect via the handler.\n\n \n let effect = continue(5)\n use num: Int &lt;- map(effect)\n num * 2\n \nSynonyms:\neffect.map\neffect map","url":"effect.html#map"},{"doc":"effect","title":"map_early","content":"pub fn map_early(\n  effect: Effect(a, b),\n  handler: fn(b) -&gt; c,\n) -&gt; Effect(a, c)\n Transforms the early return values of an effect via the handler.\n\n \n type Msg { Msg(String) }\n let effect: Effect(msg, String) = throw(&quot;some context&quot;)\n let effect: Effect(msg, Msg) = map_early(effect, Msg)\n \nSynonyms:\neffect.map_early\neffect map_early","url":"effect.html#map_early"},{"doc":"effect","title":"perform","content":"pub fn perform(\n  effect: Effect(a, b),\n  handler: fn(Result(a, b)) -&gt; c,\n) -&gt; Nil\n Executes an effect, handling both paths with a handler.\n\n \n use res: Result(msg, early) &lt;- perform(effect)\n case res {\n   Ok(value) -&gt; io.println(&quot;Success: &quot; &lt;&gt; value)\n   Error(err) -&gt; io.println(&quot;Error: &quot; &lt;&gt; err)\n }\n \nSynonyms:\neffect.perform\neffect perform","url":"effect.html#perform"},{"doc":"effect","title":"pure","content":"pub fn pure(\n  effect: Effect(a, Nothing),\n  handler: fn(a) -&gt; b,\n) -&gt; Nil\n Executes an effect that is known to be pure (cannot have early returns).\n The handler only needs to handle the success case.\n\n \n let effect: Effect(Int, early) = continue(42)\n use num: Int &lt;- pure(effect)\n num |&gt; int.to_string |&gt; io.println\n \nSynonyms:\neffect.pure\neffect pure","url":"effect.html#pure"},{"doc":"effect","title":"then","content":"pub fn then(\n  effect: Effect(a, b),\n  handler: fn(a) -&gt; Effect(c, b),\n) -&gt; Effect(c, b)\n Chains together two effects where the second effect depends on the result of the first.\n\n \n let effect = {\n   use a &lt;- then(get_user())\n   use b &lt;- then(get_posts(a.id))\n   continue(b)\n }\n \nSynonyms:\neffect.then\neffect then","url":"effect.html#then"},{"doc":"effect","title":"throw","content":"pub fn throw(value: a) -&gt; Effect(b, a)\n Creates an effect that returns early with the given value.\n\n \n let effect: Effect(any, String) = throw(&quot;Something went wrong&quot;)\n \nSynonyms:\neffect.throw\neffect throw","url":"effect.html#throw"},{"doc":"effect","title":"unbox","content":"pub fn unbox(\n  box: a,\n  unbox_fn: fn(a, fn(b) -&gt; Nil) -&gt; c,\n) -&gt; Effect(b, d)\n Creates an effect from a boxed value. Primarly used to unbox Promises but\n can work with other boxed types.\n\n \n let value: Value = ...\n let promise: Promise(Value) = promise.resolve(value)\n let effect: Effect(Value, early) = unbox(promise, promise.map)\n \nSynonyms:\neffect.unbox\neffect unbox","url":"effect.html#unbox"},{"doc":"effect","title":"wrap_option","content":"pub fn wrap_option(value: Option(a), early: b) -&gt; Effect(a, b)\n Creates an effect that wraps an option type. The Some variant continues and the\n None variant returns early with the given early value.\n\n \n let effect: Effect(Int, String) = wrap_option(Some(69), &quot;It should&#39;ve been Some tho...&quot;)\n let effect: Effect(Int, String) = wrap_option(None, &quot;This is None&quot;)\nSynonyms:\neffect.wrap_option\neffect wrap_option","url":"effect.html#wrap_option"},{"doc":"effect","title":"wrap_result","content":"pub fn wrap_result(value: Result(a, b)) -&gt; Effect(a, b)\n Creates an effect that wraps a result type. The Ok variant continues and the \n Error variant returns early.\n\n \n let effect: Effect(Int, early) = wrap_result(Ok(5))\n let effect: Effect(msg, String) = wrap_result(Error(&quot;wrong!&quot;))\n \nSynonyms:\neffect.wrap_result\neffect wrap_result","url":"effect.html#wrap_result"},{"doc":"effect","title":"effect","content":"","url":"effect.html"},{"doc":"effect/promise","title":"from_promise","content":"pub fn from_promise(\n  box: Promise(a),\n  handler: fn(a) -&gt; Effect(b, c),\n) -&gt; Effect(b, c)\n Creates an effect from a promise, operates on the inner value\n through the handler.\n \n let promise: Promise(inner)\n use inner: inner &lt;- from_promise(promise)\n effect.succeed(inner) // Effect(inner, early)\n \nSynonyms:\neffect/promise.from_promise\neffect/promise from_promise","url":"effect/promise.html#from_promise"},{"doc":"effect/promise","title":"from_promise_result","content":"pub fn from_promise_result(\n  box: Promise(Result(a, b)),\n  map_error: fn(b) -&gt; c,\n  handler: fn(a) -&gt; Effect(d, c),\n) -&gt; Effect(d, c)\n Creates an effect from a promise that contains a result.\n\n Maps the error of the result, operates on the inner\n value of the result through the handler.\n \n let promise: Promise(Result(ok, err))\n use ok: ok &lt;- from_promise_result(promise, fn (previous_error) { new_error }) // map the error here\n effect.succeed(ok) // Effect(ok, err)\n \nSynonyms:\neffect/promise.from_promise_result\neffect/promise from_promise_result","url":"effect/promise.html#from_promise_result"},{"doc":"effect/promise","title":"effect/promise","content":"","url":"effect/promise.html"}]);